@@ -1,9 +1,9 @@
 # Natural Language Toolkit: Probabilistic Chart Parsers
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Edward Loper <edloper@gradient.cis.upenn.edu>
-#         Steven Bird <sb@csse.unimelb.edu.au>
-# URL: <http://www.nltk.org/>
+# Copyright (C) 2001-2017 NLTK Project
+# Author: Edward Loper <edloper@gmail.com>
+#         Steven Bird <stevenbird1@gmail.com>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -29,6 +29,7 @@ The ``BottomUpProbabilisticChartParser`` constructor has an optional
 argument beam_size.  If non-zero, this controls the size of the beam
 (aka the edge queue).  This option is most useful with InsideChartParser.
 """
+from __future__ import print_function, unicode_literals
 
 ##//////////////////////////////////////////////////////
 ##  Bottom-Up PCFG Chart Parser
@@ -37,11 +38,13 @@ argument beam_size.  If non-zero, this controls the size of the beam
 # [XX] This might not be implemented quite right -- it would be better
 # to associate probabilities with child pointer lists.
 
+from functools import reduce
 from nltk.tree import Tree, ProbabilisticTree
-from nltk.grammar import Nonterminal, WeightedGrammar
+from nltk.grammar import Nonterminal, PCFG
 
 from nltk.parse.api import ParserI
 from nltk.parse.chart import Chart, LeafEdge, TreeEdge, AbstractChartRule
+from nltk.compat import python_2_unicode_compatible
 
 # Probabilistic edges
 class ProbabilisticLeafEdge(LeafEdge):
@@ -49,15 +52,13 @@ class ProbabilisticLeafEdge(LeafEdge):
 
 class ProbabilisticTreeEdge(TreeEdge):
     def __init__(self, prob, *args, **kwargs):
-        self._prob = prob
         TreeEdge.__init__(self, *args, **kwargs)
+        self._prob = prob
+        # two edges with different probabilities are not equal.
+        self._comparison_key = (self._comparison_key, prob)
 
     def prob(self): return self._prob
 
-    def __cmp__(self, other):
-        if self._prob != other.prob(): return -1
-        return TreeEdge.__cmp__(self, other)
-
     @staticmethod
     def from_production(production, index, p):
         return ProbabilisticTreeEdge(p, (index, index), production.lhs(),
@@ -66,7 +67,7 @@ class ProbabilisticTreeEdge(TreeEdge):
 # Rules using probabilistic edges
 class ProbabilisticBottomUpInitRule(AbstractChartRule):
     NUM_EDGES=0
-    def apply_iter(self, chart, grammar):
+    def apply(self, chart, grammar):
         for index in range(chart.num_leaves()):
             new_edge = ProbabilisticLeafEdge(chart.leaf(index), index)
             if chart.insert(new_edge, ()):
@@ -74,7 +75,7 @@ class ProbabilisticBottomUpInitRule(AbstractChartRule):
 
 class ProbabilisticBottomUpPredictRule(AbstractChartRule):
     NUM_EDGES=1
-    def apply_iter(self, chart, grammar, edge):
+    def apply(self, chart, grammar, edge):
         if edge.is_incomplete(): return
         for prod in grammar.productions():
             if edge.lhs() == prod.rhs()[0]:
@@ -84,10 +85,10 @@ class ProbabilisticBottomUpPredictRule(AbstractChartRule):
 
 class ProbabilisticFundamentalRule(AbstractChartRule):
     NUM_EDGES=2
-    def apply_iter(self, chart, grammar, left_edge, right_edge):
+    def apply(self, chart, grammar, left_edge, right_edge):
         # Make sure the rule is applicable.
         if not (left_edge.end() == right_edge.start() and
-                left_edge.next() == right_edge.lhs() and
+                left_edge.nextsym() == right_edge.lhs() and
                 left_edge.is_incomplete() and right_edge.is_complete()):
             return
 
@@ -107,27 +108,29 @@ class ProbabilisticFundamentalRule(AbstractChartRule):
         # If we changed the chart, then generate the edge.
         if changed_chart: yield new_edge
 
+@python_2_unicode_compatible
 class SingleEdgeProbabilisticFundamentalRule(AbstractChartRule):
     NUM_EDGES=1
 
     _fundamental_rule = ProbabilisticFundamentalRule()
 
-    def apply_iter(self, chart, grammar, edge1):
+    def apply(self, chart, grammar, edge1):
         fr = self._fundamental_rule
         if edge1.is_incomplete():
             # edge1 = left_edge; edge2 = right_edge
             for edge2 in chart.select(start=edge1.end(), is_complete=True,
-                                     lhs=edge1.next()):
-                for new_edge in fr.apply_iter(chart, grammar, edge1, edge2):
+                                     lhs=edge1.nextsym()):
+                for new_edge in fr.apply(chart, grammar, edge1, edge2):
                     yield new_edge
         else:
             # edge2 = left_edge; edge1 = right_edge
             for edge2 in chart.select(end=edge1.start(), is_complete=False,
-                                     next=edge1.lhs()):
-                for new_edge in fr.apply_iter(chart, grammar, edge2, edge1):
+                                      nextsym=edge1.lhs()):
+                for new_edge in fr.apply(chart, grammar, edge2, edge1):
                     yield new_edge
 
-    def __str__(self): return 'Fundamental Rule'
+    def __str__(self):
+        return 'Fundamental Rule'
 
 class BottomUpProbabilisticChartParser(ParserI):
     """
@@ -170,8 +173,8 @@ class BottomUpProbabilisticChartParser(ParserI):
             and higher numbers will produce more verbose tracing
             output.
         """
-        if not isinstance(grammar, WeightedGrammar):
-            raise ValueError("The grammar must be probabilistic WeightedGrammar")
+        if not isinstance(grammar, PCFG):
+            raise ValueError("The grammar must be probabilistic PCFG")
         self._grammar = grammar
         self.beam_size = beam_size
         self._trace = trace
@@ -193,7 +196,7 @@ class BottomUpProbabilisticChartParser(ParserI):
         self._trace = trace
 
     # TODO: change this to conform more with the standard ChartParser
-    def nbest_parse(self, tokens, n=None):
+    def parse(self, tokens):
         self._grammar.check_coverage(tokens)
         chart = Chart(list(tokens))
         grammar = self._grammar
@@ -203,14 +206,14 @@ class BottomUpProbabilisticChartParser(ParserI):
         bu = ProbabilisticBottomUpPredictRule()
         fr = SingleEdgeProbabilisticFundamentalRule()
 
-        # Our queue!
+        # Our queue
         queue = []
 
         # Initialize the chart.
-        for edge in bu_init.apply_iter(chart, grammar):
+        for edge in bu_init.apply(chart, grammar):
             if self._trace > 1:
-                print '  %-50s [%s]' % (chart.pp_edge(edge,width=2),
-                                        edge.prob())
+                print('  %-50s [%s]' % (chart.pretty_format_edge(edge,width=2),
+                                        edge.prob()))
             queue.append(edge)
 
         while len(queue) > 0:
@@ -224,15 +227,15 @@ class BottomUpProbabilisticChartParser(ParserI):
             # Get the best edge.
             edge = queue.pop()
             if self._trace > 0:
-                print '  %-50s [%s]' % (chart.pp_edge(edge,width=2),
-                                        edge.prob())
+                print('  %-50s [%s]' % (chart.pretty_format_edge(edge,width=2),
+                                        edge.prob()))
 
             # Apply BU & FR to it.
             queue.extend(bu.apply(chart, grammar, edge))
             queue.extend(fr.apply(chart, grammar, edge))
 
         # Get a list of complete parses.
-        parses = chart.parses(grammar.start(), ProbabilisticTree)
+        parses = list(chart.parses(grammar.start(), ProbabilisticTree))
 
         # Assign probabilities to the trees.
         prod_probs = {}
@@ -242,19 +245,19 @@ class BottomUpProbabilisticChartParser(ParserI):
             self._setprob(parse, prod_probs)
 
         # Sort by probability
-        parses.sort(lambda a,b: cmp(b.prob(), a.prob()))
+        parses.sort(reverse=True, key=lambda tree: tree.prob())
 
-        return parses[:n]
+        return iter(parses)
 
     def _setprob(self, tree, prod_probs):
         if tree.prob() is not None: return
 
         # Get the prob of the CFG production.
-        lhs = Nonterminal(tree.node)
+        lhs = Nonterminal(tree.label())
         rhs = []
         for child in tree:
             if isinstance(child, Tree):
-                rhs.append(Nonterminal(child.node))
+                rhs.append(Nonterminal(child.label()))
             else:
                 rhs.append(child)
         prob = prod_probs[lhs, tuple(rhs)]
@@ -291,7 +294,7 @@ class BottomUpProbabilisticChartParser(ParserI):
             split = len(queue)-self.beam_size
             if self._trace > 2:
                 for edge in queue[:split]:
-                    print '  %-50s [DISCARDED]' % chart.pp_edge(edge,2)
+                    print('  %-50s [DISCARDED]' % chart.pretty_format_edge(edge,2))
             del queue[:split]
 
 class InsideChartParser(BottomUpProbabilisticChartParser):
@@ -325,7 +328,7 @@ class InsideChartParser(BottomUpProbabilisticChartParser):
         :type chart: Chart
         :rtype: None
         """
-        queue.sort(lambda e1,e2:cmp(e1.prob(), e2.prob()))
+        queue.sort(key=lambda edge: edge.prob())
 
 # Eventually, this will become some sort of inside-outside parser:
 # class InsideOutsideParser(BottomUpProbabilisticChartParser):
@@ -348,12 +351,11 @@ class InsideChartParser(BottomUpProbabilisticChartParser):
 #         self._bestp = bestp
 #         for (k,v) in self._bestp.items(): print k,v
 #
-#     def _cmp(self, e1, e2):
-#         return cmp(e1.structure()[PROB]*self._bestp[e1.lhs()],
-#                    e2.structure()[PROB]*self._bestp[e2.lhs()])
+#     def _sortkey(self, edge):
+#         return edge.structure()[PROB] * self._bestp[edge.lhs()]
 #
 #     def sort_queue(self, queue, chart):
-#         queue.sort(self._cmp)
+#         queue.sort(key=self._sortkey)
 
 import random
 class RandomChartParser(BottomUpProbabilisticChartParser):
@@ -381,7 +383,7 @@ class LongestChartParser(BottomUpProbabilisticChartParser):
     """
     # Inherit constructor
     def sort_queue(self, queue, chart):
-        queue.sort(lambda e1,e2: cmp(e1.length(), e2.length()))
+        queue.sort(key=lambda edge: edge.length())
 
 ##//////////////////////////////////////////////////////
 ##  Test Code
@@ -395,27 +397,64 @@ def demo(choice=None, draw_parses=None, print_parses=None):
     summary of the results are displayed.
     """
     import sys, time
-    from nltk import tokenize, toy_pcfg1, toy_pcfg2
+    from nltk import tokenize
     from nltk.parse import pchart
 
     # Define two demos.  Each demo has a sentence and a grammar.
+    toy_pcfg1 = PCFG.fromstring("""
+    S -> NP VP [1.0]
+    NP -> Det N [0.5] | NP PP [0.25] | 'John' [0.1] | 'I' [0.15]
+    Det -> 'the' [0.8] | 'my' [0.2]
+    N -> 'man' [0.5] | 'telescope' [0.5]
+    VP -> VP PP [0.1] | V NP [0.7] | V [0.2]
+    V -> 'ate' [0.35] | 'saw' [0.65]
+    PP -> P NP [1.0]
+    P -> 'with' [0.61] | 'under' [0.39]
+    """)
+
+    toy_pcfg2 = PCFG.fromstring("""
+    S    -> NP VP         [1.0]
+    VP   -> V NP          [.59]
+    VP   -> V             [.40]
+    VP   -> VP PP         [.01]
+    NP   -> Det N         [.41]
+    NP   -> Name          [.28]
+    NP   -> NP PP         [.31]
+    PP   -> P NP          [1.0]
+    V    -> 'saw'         [.21]
+    V    -> 'ate'         [.51]
+    V    -> 'ran'         [.28]
+    N    -> 'boy'         [.11]
+    N    -> 'cookie'      [.12]
+    N    -> 'table'       [.13]
+    N    -> 'telescope'   [.14]
+    N    -> 'hill'        [.5]
+    Name -> 'Jack'        [.52]
+    Name -> 'Bob'         [.48]
+    P    -> 'with'        [.61]
+    P    -> 'under'       [.39]
+    Det  -> 'the'         [.41]
+    Det  -> 'a'           [.31]
+    Det  -> 'my'          [.28]
+    """)
+
     demos = [('I saw John with my telescope', toy_pcfg1),
              ('the boy saw Jack with Bob under the table with a telescope',
               toy_pcfg2)]
 
     if choice is None:
         # Ask the user which demo they want to use.
-        print
+        print()
         for i in range(len(demos)):
-            print '%3s: %s' % (i+1, demos[i][0])
-            print '     %r' % demos[i][1]
-            print
-        print 'Which demo (%d-%d)? ' % (1, len(demos)),
+            print('%3s: %s' % (i+1, demos[i][0]))
+            print('     %r' % demos[i][1])
+            print()
+        print('Which demo (%d-%d)? ' % (1, len(demos)), end=' ')
         choice = int(sys.stdin.readline().strip())-1
     try:
         sent, grammar = demos[choice]
     except:
-        print 'Bad sentence number'
+        print('Bad sentence number')
         return
 
     # Tokenize the sentence.
@@ -436,49 +475,48 @@ def demo(choice=None, draw_parses=None, print_parses=None):
     num_parses = []
     all_parses = {}
     for parser in parsers:
-        print '\ns: %s\nparser: %s\ngrammar: %s' % (sent,parser,grammar)
+        print('\ns: %s\nparser: %s\ngrammar: %s' % (sent,parser,grammar))
         parser.trace(3)
         t = time.time()
-        parses = parser.nbest_parse(tokens)
+        parses = list(parser.parse(tokens))
         times.append(time.time()-t)
-        if parses: p = reduce(lambda a,b:a+b.prob(), parses, 0)/len(parses)
-        else: p = 0
+        p = (reduce(lambda a,b:a+b.prob(), parses, 0)/len(parses) if parses else 0)
         average_p.append(p)
         num_parses.append(len(parses))
         for p in parses: all_parses[p.freeze()] = 1
 
     # Print some summary statistics
-    print
-    print '       Parser      Beam | Time (secs)   # Parses   Average P(parse)'
-    print '------------------------+------------------------------------------'
+    print()
+    print('       Parser      Beam | Time (secs)   # Parses   Average P(parse)')
+    print('------------------------+------------------------------------------')
     for i in range(len(parsers)):
-        print '%18s %4d |%11.4f%11d%19.14f' % (parsers[i].__class__.__name__,
+        print('%18s %4d |%11.4f%11d%19.14f' % (parsers[i].__class__.__name__,
                                              parsers[i].beam_size,
-                                             times[i],num_parses[i],average_p[i])
+                                             times[i],num_parses[i],average_p[i]))
     parses = all_parses.keys()
     if parses: p = reduce(lambda a,b:a+b.prob(), parses, 0)/len(parses)
     else: p = 0
-    print '------------------------+------------------------------------------'
-    print '%18s      |%11s%11d%19.14f' % ('(All Parses)', 'n/a', len(parses), p)
+    print('------------------------+------------------------------------------')
+    print('%18s      |%11s%11d%19.14f' % ('(All Parses)', 'n/a', len(parses), p))
 
     if draw_parses is None:
         # Ask the user if we should draw the parses.
-        print
-        print 'Draw parses (y/n)? ',
+        print()
+        print('Draw parses (y/n)? ', end=' ')
         draw_parses = sys.stdin.readline().strip().lower().startswith('y')
     if draw_parses:
         from nltk.draw.tree import draw_trees
-        print '  please wait...'
+        print('  please wait...')
         draw_trees(*parses)
 
     if print_parses is None:
         # Ask the user if we should print the parses.
-        print
-        print 'Print parses (y/n)? ',
+        print()
+        print('Print parses (y/n)? ', end=' ')
         print_parses = sys.stdin.readline().strip().lower().startswith('y')
     if print_parses:
         for parse in parses:
-            print parse
+            print(parse)
 
 if __name__ == '__main__':
     demo()
